/*
 * xml-dtc provides an XML-DTC implementation as a CGI program
 * Copyright (C) 2003-2004 Mike Hommey <mh@glandium.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#define _GNU_SOURCE
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <xml-dtc.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

extern char **environ;

int register_headers(xml_dtc_request *request) {
  char *tmp;
  char **env = environ;

  if (env)
    while (*env) {
      if (strncmp(*env, "HTTP_", 5) == 0) {
	tmp = index(*env, '=');
	if (tmp) {
	  *tmp = 0;
	  xml_dtc_request_set_header(request, *env + 5, tmp + 1);
	  *tmp = '=';
	}
      }
      env++;
    }
  return 0;
}

int register_parameters(xml_dtc_request *request) {
  char *query_string = getenv("QUERY_STRING");

  return xml_dtc_request_set_parameters_from_query_string(request, query_string);
}

char *get_map_file_name() {
  char *map;
  int fd;
  struct stat filestat;

  if ((stat("sitemap.xmap", &filestat) == 0) && (S_ISREG(filestat.st_mode)))
    return strdup("sitemap.xmap");

  map = getenv("XML_DTC_SITEMAP");
  if (map)
    return strdup(map);

  fd = open("/etc/xml-dtc/cgi.conf", O_RDONLY);
  if (fd != -1) {
    char buf[1024];
    int r;

    r = read(fd, buf, 1024);
    close(fd);
    if (r > 0) {
      char *i;
      buf[1023] = 0;
      i = index(buf, '\n');
      if (i)
	return strndup(buf, i - buf);
      return strdup(buf);
    }
  }
  return strdup("sitemap.xmap");
}

int main(void) {
  xml_dtc_sitemap *sitemap;
  xml_dtc_request *request;
  char *uri, *path_info, *map;
  int i;

  if ((path_info = getenv("PATH_INFO")) == NULL) {
    path_info = getenv("SCRIPT_NAME");
    printf("Status: 302 Found\nLocation: %s/\n\n", path_info?path_info:"");
    return 0;
  }

  if ((uri = getenv("REQUEST_URI"))) {
    char *tmp;
    tmp = index(uri, '?');
    if (tmp)
      *tmp = 0;
  }

  xml_dtc_init(NULL);
  xml_dtc_init_modules();
  request = xml_dtc_request_new(xml_dtc.resource, getenv("REQUEST_METHOD"), getenv("REQUEST_URI"), path_info);
  request->register_headers = register_headers;
  request->register_parameters = register_parameters;
  xml_dtc_resource_set_msg_verbosity(xml_dtc.resource, XML_DTC_MSG_ERROR);
  // xml_dtc_resource_set_verbosity(xml_dtc.resource, XML_DTC_MSG_DEBUG);


  map = get_map_file_name();
  sitemap = xml_dtc_sitemap_new_from_file(xml_dtc.resource, map);
  free(map);

  xml_dtc_sitemap_request(sitemap, request);

  if (request->status == 302) {
    write(1, "Status: 302 Found\n", 18);
  } else {
    if (request->error.type != NULL) {
      write(1, "Status: 404 Not Found\n", 22);
    }
    write(1, "Content-type: ", 14);
    if (request->mime_type)
      write(1, request->mime_type, strlen(request->mime_type));
    else
      write(1, "text/plain", 10);
    if (request->encoding) {
      write(1, "; charset=", 10);
      write(1, request->encoding, strlen(request->encoding));
    }
    write(1, "\n", 1);
  }
  if (request->out_headers)
    for(i = 0; i < request->out_headers->num; i++) {
      fprintf(stdout, "%s: %s\n", request->out_headers->list[i].key, (char *) request->out_headers->list[i].element);
      fflush(stdout);
    }
  if (request->newcookies)
    for (i = 0; i < request->newcookies->num; i++) {
      fprintf(stdout, "Set-Cookie: %s=%s\n", request->newcookies->list[i].key, (char *)request->newcookies->list[i].element);
      fflush(stdout);
    }
  if ((request->status != 302) && (request->content_length >= 0)) {
    fprintf(stdout, "Content-length: %d\n\n", request->content_length);
    fflush(stdout);
    write(1, request->result, request->content_length);
  } else
    fprintf(stdout,"\n%s", request->result);

  xml_dtc_sitemap_free(sitemap);

  xml_dtc_request_free(request);

  xml_dtc_cleanup();

  return 0;
}
