/*
 * This module provides readers to the XML-DTC framework
 * Copyright (C) 2003-2004 Mike Hommey <mh@glandium.org>
 * 
 * This module is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This module is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this module; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include <modules.h>
#include <xml-dtc.h>
#include <mappings.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

typedef struct _xml_dtc_reader_caller_data xml_dtc_reader_caller_data;

struct _xml_dtc_reader_caller_data {
  xmlChar *src, *mime_type;
};

static xml_dtc_component *xml_dtc_handler_reader_callback(xml_dtc_handler *handler, xml_dtc_hash *handlers, xml_dtc_sitemap *sitemap, xmlNodePtr node) {
  xmlChar *attr;
  xml_dtc_component *result;

  attr = xmlGetProp(node, "name");
  result = xml_dtc_component_new(sitemap->resource, attr, (char *)node->name);
  result->handle = handler->_private;
  free(attr);

  xml_dtc_sitemap_add(sitemap, result);
  return (xml_dtc_component *)handler;
}

int _xml_dtc_reader_caller_free(xml_dtc_reader_caller_data *data) {
  if (data) {
    if (data->src)
      xmlFree(data->src);
    if (data->mime_type)
      xmlFree(data->mime_type);
    free(data);
  }
  return 0;
}

static xml_dtc_component *xml_dtc_handler_read_callback(xml_dtc_handler *handler, xml_dtc_hash *handlers, xml_dtc_sitemap *sitemap, xmlNodePtr node) {
  xml_dtc_caller *result;
  xml_dtc_reader_caller_data *data;
  result = xml_dtc_caller_new(sitemap->resource, "reader", "reader");
  data = malloc(sizeof(xml_dtc_reader_caller_data));
  memset(data, 0, sizeof(xml_dtc_reader_caller_data));
  result->component.name = xmlGetProp(node, "type");
  result->_private = data;
  result->free_private = (int (*) (void *))_xml_dtc_reader_caller_free;
  data->src = xmlGetProp(node, "src");
  data->mime_type = xmlGetProp(node, "mime-type");
  
  return (xml_dtc_component *)result;
}

static int xml_dtc_read_file(xml_dtc_caller *caller, xml_dtc_request *request) {
  int fd;
  char *buf, *filename;
  struct stat file_stat;
  xml_dtc_reader_caller_data *data = caller->_private;

  filename = xml_dtc_resolve_mappings(request->mappings, data->src);
  xml_dtc_sitemap_resolve_relative_path(request->sitemap, &filename);

  fd = open(filename, O_RDONLY);
  free(filename);
  if (fd != -1) {
    if (fstat(fd, &file_stat) == 0) {
      if (S_ISDIR(file_stat.st_mode)) {
	xml_dtc_request_set_error(request, caller->component.object, "resource-not-found", "Resource not found", "Cannot open file.");
	return -1;
      }
#if 0
      //      if (request->write) {
	int r;
	request->set_mime_type(request->write_context, caller->mime_type, NULL);
	request->set_content_length(request->write_context, file_stat.st_size);
	request->send_http_header(request->write_context);
	buf = malloc(409600);
	while ((r = read(fd, buf, 409600))) {
	  request->write(request->write_context, buf, r);
	}
	request->content_type = null;
	//} else {
#endif
	buf = malloc(file_stat.st_size);
	read(fd, buf, file_stat.st_size);
	request->result = buf;
	request->content_length = file_stat.st_size;
	request->mime_type = data->mime_type;
	//      }
    }
    close(fd);
  } else {
    xml_dtc_request_set_error(request, caller->component.object, "resource-not-found", "Resource not found", "Cannot open file.");
  }

  return 0;
}

static int xml_dtc_readers_module_register(void) {
  return 0;
}

static const xml_dtc_caller_def xml_dtc_readers_types_def[] = {
  { "read", "reader", (xml_dtc_handler_callback) xml_dtc_handler_read_callback },
  { NULL }
};

static const xml_dtc_component_def xml_dtc_readers_def[] = {
  { "resource", "reader", xml_dtc_read_file, (xml_dtc_handler_callback) xml_dtc_handler_reader_callback },
  { NULL }
};

const xml_dtc_module xml_dtc_readers_module = {
  XML_DTC_MODULE_STANDARD_STUFF,
  xml_dtc_readers_module_register,
  NULL,
  xml_dtc_readers_types_def,
  xml_dtc_readers_def
};
